package org.btcollider.cnc.keysrv;

import java.util.concurrent.TimeUnit;
import org.btcollider.cnc.keysrv.keytree.KeyTreeServer;
import org.btcollider.cnc.keysrv.multistage.MultiStagedServer;

public class KSFactory {
  private KSFactory() {}

  public static KeyServer build(int index, int depth, long maxWorkSpan) {
    return build(index, depth, maxWorkSpan, TimeUnit.MILLISECONDS);
  }

  public static KeyServer build(int index, int depth, long maxWorkSpan, TimeUnit timeUnit) {
    return build(index, depth, maxWorkSpan, timeUnit, 0);
  }

  public static KeyServer build(int index, int depth, long maxWorkSpan, TimeUnit timeUnit,
      float pruneFraction) {

    assert (pruneFraction >= 0.0 && pruneFraction <= 1.0);

    long maxKey = ((long) 1 << index) | (((long) 1 << index) - 1);
    long minKey = ((long) 1 << index);

    long pruneKey = (long) Math.floor((maxKey - minKey) * pruneFraction) + minKey;

    return build(index, depth, maxWorkSpan, timeUnit, pruneKey);
  }

  public static KeyServer build(int index, int depth, long maxWorkSpan, TimeUnit timeUnit,
      long pruneKey) {
    if (depth == 25) {
      return new KeyTreeServer(index, depth, timeUnit.toMillis(maxWorkSpan), pruneKey);
    } else {
      return new MultiStagedServer(index, depth, timeUnit.toMillis(maxWorkSpan), pruneKey);
    }

  }
}
