package org.btcollider.cnc.keysrv.multistage;

import java.util.HashMap;
import java.util.Map.Entry;
import org.btcollider.cnc.dto.KeyRange;

/**
 * The MultiStagedServer may switch partitions while there are still KeyRanges from the old
 * partition that are currently worked on.
 *
 * This buffer stores the state of unfinished KeyRanges from old partitions in order to handle them
 * correctly.
 *
 * Aborted KeyRanges from this Buffer should be always preferred to getting new ranges.
 *
 * @author armin
 */
public class KeyRangeBuffer {
  private HashMap<KeyRange, Long> keyRanges;
  private Long maxWorkSpan;

  public KeyRangeBuffer(Long maxWorkSpan) {
    this.keyRanges = new HashMap<>();
    this.maxWorkSpan = maxWorkSpan;
  }

  public void setInWork(KeyRange keyRange) {
    this.keyRanges.put(keyRange, System.currentTimeMillis());
  }

  public void setSearched(KeyRange keyRange) {
    keyRanges.computeIfPresent(keyRange, (kr, iw) -> null);
  }

  public KeyRange getRange() {
    long curTime = System.currentTimeMillis();

    KeyRange retracedRange = null;
    for (Entry<KeyRange, Long> kr : keyRanges.entrySet()) {
      if ((curTime - kr.getValue()) >= maxWorkSpan) {
        retracedRange = kr.getKey();
        break;
      }
    }

    return retracedRange;
  }
}
